/* StreamFeatureTable.java
 *
 * created: Wed Sep 15 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1999  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/uk/ac/sanger/pathogens/embl/StreamFeatureTable.java,v 1.2 1999/09/28 12:34:01 kmr Exp $
 */

package uk.ac.sanger.pathogens.embl;

import collections.*;

import java.util.Vector;
import java.io.*;
import java.util.NoSuchElementException;

/**
 *  This object contains all the features in an entry.  This object can read
 *  and write itself to a stream.
 *
 *  @author Kim Rutherford
 *  @version $Id: StreamFeatureTable.java,v 1.2 1999/09/28 12:34:01 kmr Exp $
 */

abstract class StreamFeatureTable extends FeatureTable {
  /**
   *  Create a new empty StreamFeatureTable object.
   **/
  StreamFeatureTable () {

  }

  /**
   *  Create a new StreamFeatureTable object from the given FeatureTable.
   **/
  StreamFeatureTable (final FeatureTable feature_table) {
    final FeatureVector source_features = feature_table.getSourceFeatures ();

    for (int i = 0 ; i < source_features.size () ; ++i) {
      addSourceFeature (source_features.elementAt (i));
    }

    final FeatureEnumeration feature_enum = feature_table.features ();

    while (feature_enum.hasMoreFeatures ()) {
      final Feature new_feature = feature_enum.nextFeature ();

      add (new_feature);
    }
  }

  /**
   *  Create a new StreamFeatureTable object from an stream.
   *  @param in_stream The stream to read from.  When the constructor returns
   *    the stream will at the next line after the feature table.
   *  @exception InvalidRelationException Thrown if this Feature cannot contain
   *    the given Qualifier.
   **/
  StreamFeatureTable (LinePushBackReader in_stream)
      throws IOException, InvalidRelationException {
    Feature new_feature;

    while ((new_feature = readFeatureFromStream (in_stream)) != null) {
      if (new_feature.getKey ().equals ("source")) {
        addSourceFeature (new_feature);
      } else {
        // add it at the correct position in the StreamFeatureTable
        add (new_feature);
      }
      Thread.yield ();
    }
  }

  /**
   *  Read and return a StreamFeature.  (called by
   *  StreamFeatureTable.readFeatureFromStream ()).
   *  @param in_stream The stream to read from.  When the constructor returns
   *    the stream will at the next line after the feature table.
   *  @exception InvalidRelationException Thrown if this Feature cannot contain
   *    the given Qualifier.
   **/
  abstract StreamFeature readFeatureFromStream (LinePushBackReader in_stream)
      throws IOException, InvalidRelationException;

  /**
   *  Write this StreamFeatureTable to the given stream.  The output will be in EMBL
   *  format (but see the diana_extensions argument).
   *  @param writer The stream to write to.
   *  @param diana_extensions If true then the any diana additions to the embl
   *    file format will be included in the output, otherwise they will be
   *    removed.
   **/
  public void writeToStream (final Writer writer,
                             final boolean diana_extensions)
      throws IOException {

    for (int i = 0 ; i < getSourceFeatures ().size () ; ++i) {
      final StreamFeature this_feature =
        (StreamFeature) getSourceFeatures ().elementAt (i);

      this_feature.writeToStream (writer, diana_extensions);
    }

    final FeatureEnumeration feature_enum = features ();

    while (feature_enum.hasMoreFeatures ()) {
      final StreamFeature this_feature =
        (StreamFeature) feature_enum.nextFeature ();

      this_feature.writeToStream (writer, diana_extensions);
    }
  }
}
